import greenfoot.*;  // (World, Actor, GreenfootImage, and Greenfoot)
import java.awt.Color;
import java.util.Random;

/**
 * 鋼琴的琴鍵。
 * 
 * @author 依瑪貓
 * @version 2012/12/3
 */
public class Key extends Actor {
    
    /** 亂數參數一。 */
    private static final int RANDOM_PARAM_A = 2123417911;
    
    /** 亂數參數二。 */
    private static final int RANDOM_PARAM_B = 1846175159;
    
    /** 琴鍵的音符。 */
    private String note = null;
    
    /** 琴鍵的鍵盤按鍵。 */
    private String key = null;
    
    /** 琴鍵的類別。 */
    private String type = null;
    
    /** 有沒有壓下琴鍵。 */
    private boolean isDown = false;
    
    /** 琴鍵的閃閃星光。 */
    private Spark spark = null;
    
    /** 是否正在自動彈奏中。 */
    private boolean isAutoPlay = false;
    
    /** 自動彈奏時，鍵盤壓下的計時器。 */
    private int beatCount = 0;
    
    /** 亂數種子。 */
    private Random random = new Random();
    
    /**
     * 依琴鍵的鍵盤按鍵，和琴鍵的聲音，建立琴鍵。
     * 
     * @param noteName 琴鍵的音符
     * @param keyName  琴鍵的鍵盤按鍵
     * @param keyType    琴鍵的類別
     */
    public Key(String noteName, String keyName, String keyType) {
        key = keyName;
        note = noteName;
        type = keyType;
        setImage(type + "-key.png");
    }
    
    /**
     * 琴鍵加進鋼琴後，要執行的動作。
     * 
     */
    public void addedToWorld(World world) {
        addSpark();
    }
    
    /**
     * 琴鍵每回合執行的動作。
     * 
     */
    public void act() {
        if (!isDown && (Greenfoot.isKeyDown(key) || isAutoPlay))
        {
            setImage(type + "-key-down.png");
            isDown = true;
            play();
        }
        if (isDown && !(Greenfoot.isKeyDown(key) || isAutoPlay))
        {
            setImage(type + "-key.png");
            isDown = false;
        }
        if (isAutoPlay) {
            beatCount--;
            if (beatCount == 0) {
                isAutoPlay = false;
            }
        }
    }
    
    /**
     * 自動演奏琴鍵的音符。
     * 
     */
    public void autoPlay() {
        isAutoPlay = true;
        beatCount = 4;
    }
    
    /**
     * 演奏琴鍵的音符。
     * 
     */
    private void play() {
        Greenfoot.playSound(note + ".wav");
        spark.sparkle();
    }
    
    /**
     * 加上點點星光。
     * 
     */
    private void addSpark() {
        setSeed();
        GreenfootImage image = getImage();
        World world = getWorld();
        int width = image.getWidth();
        int height = getY() - image.getHeight() / 2;
        int x = getX();
        int y = height / 2;
        int red = 128 + random.nextInt(128);
        int green = 128 + random.nextInt(128);
        int blue = 128 + random.nextInt(128);
        Color color = new Color(red, green, blue);
        
        spark = new Spark(color, width, height);
        world.addObject(spark, x, y);
    }
    
    /**
     * 初始化亂數種子。
     * 
     */
    private void setSeed() {
        long seed = 0;
        for (int i = 0; i < note.length(); i++) {
            seed = seed * 256 + note.codePointAt(i);
        }
        seed = (seed * RANDOM_PARAM_A) % RANDOM_PARAM_B;
        random.setSeed(seed);
    }
}