import greenfoot.*;  // (World, Actor, GreenfootImage, and Greenfoot)
import java.awt.Color;
import java.util.Hashtable;
import java.util.Map;
import java.util.Random;

/**
 * 用鍵盤彈奏的鋼琴。
 * 
 * @author 依瑪貓
 * @version 2012/12/3
 */
public class Piano extends World {
    
    /** 鋼琴白鍵的鍵盤按鍵。 */
    private String[] whiteKeys =
        { "a", "s", "d", "f", "g", "h", "j", "k", "l", ";", "'",
            "\\" };
    
    /** 鋼琴白鍵的音符。 */
    private String[] whiteNotes =
        { "3c", "3d", "3e", "3f", "3g", "3a", "3b", "4c", "4d", "4e",
            "4f", "4g" };
    
    /** 鋼琴黑鍵的鍵盤按鍵。 */
    private String[] blackKeys =
        { "w", "e", "", "t", "y", "u", "", "o", "p", "", "]" };
    
    /** 鋼琴黑鍵的音符。 */
    private String[] blackNotes =
        { "3c#", "3d#", "", "3f#", "3g#", "3a#", "", "4c#", "4d#", "", "4f#" };
    
    /** 鋼琴琴鍵。 */
    private Map<String,Key> keys = new Hashtable<String,Key>();
    
    /** 小星星。 */
    private String twinkle[] =
        { "3c", "3c", "3g", "3g", "3a", "3a", "3g", "", "3f", "3f", "3e", "3e", "3d", "3d", "3c", "",
          "3g", "3g", "3f", "3f", "3e", "3e", "3d", "", "3g", "3g", "3f", "3f", "3e", "3e", "3d", "",
          "3c", "3c", "3g", "3g", "3a", "3a", "3g", "", "3f", "3f", "3e", "3e", "3d", "3d", "3c", "" };
    
    /** 是不是自動彈奏中。 */
    private boolean isPlaying = false;
    
    /** 自動彈奏目前的拍子數。 */
    private int beat = 0;
    
    /**
     * 製作鋼琴。
     * 
     */
    public Piano() {
        super(600, 600, 1);
        paintBackground();
        //getNextPrime();
        makeKeys();
        addObject(new PlayButton(), 600 - 30, 600 - 30);
        
        /*
        java.util.List<Spark> sparks = getObjects(Spark.class);
        for (Spark spark : sparks) {
            spark.show();
        }
        */
    }
    
    /**
     * 繪製背景。
     * 
     */
    private void paintBackground() {
        GreenfootImage image = getBackground();
        image.setColor(Color.BLACK);
        image.fill();
        addStars(500);
    }
    
    /**
     * 加上星星。
     * 
     * @param number 星星的數目。
     */
    private void addStars(int number) {
        GreenfootImage image = getBackground();
        for (int i = 0; i < number; i++) {
            int red = Greenfoot.getRandomNumber(256);
            int green = Greenfoot.getRandomNumber(256);
            int blue = Greenfoot.getRandomNumber(256);
            Color color = new Color(red, green, blue);
            int x = Greenfoot.getRandomNumber(getWidth() - 1);
            int y = Greenfoot.getRandomNumber(getHeight() - 1);
            image.setColor(color);
            image.fillOval(x, y, 2, 2);
        }
    }
    
    /**
     * 鋼琴每回合執行的動作。
     * 
     */
    public void act() {
        if (isPlaying) {
            if (beat % 32 == 0) {
                int current = beat / 32;
                if (!twinkle[current].equals("")) {
                    Key key = keys.get(twinkle[current]);
                    key.autoPlay();
                }
                if (current == twinkle.length - 1) {
                    isPlaying = false;
                }
            }
            beat++;
        }
    }
    
    /**
     * 自動彈奏小星星。
     * 
     */
    public void playTwinkle() {
        isPlaying = true;
        beat = 0;
    }
    
    /**
     * 製作鋼琴的琴鍵。
     * 
     */
    private void makeKeys() {
        GreenfootImage whiteKeyImage = new GreenfootImage("white-key.png");
        GreenfootImage blackKeyImage = new GreenfootImage("black-key.png");
        int keyWidth = whiteKeyImage.getWidth();
        int whiteKeyHeight = whiteKeyImage.getHeight();
        int blackKeyHeight = blackKeyImage.getHeight();
        int x = -1, y = -1;
        
        y = getHeight() - 30 - whiteKeyHeight / 2;
        for (int i = 0; i < whiteKeys.length; i++) {
            Key key = new Key(whiteNotes[i], whiteKeys[i], "white");
            keys.put(whiteNotes[i], key);
            x = (getWidth() - keyWidth * (whiteKeys.length - 1)) / 2 + i * keyWidth;
            addObject(key, x, y);
        }
        
        y = getHeight() - 30 - whiteKeyHeight + blackKeyHeight / 2;
        for (int i = 0; i < blackKeys.length; i++) {
            if (!blackKeys[i].equals("")) {
                Key key = new Key(blackNotes[i], blackKeys[i], "black");
                keys.put(blackNotes[i], key);
                x = (getWidth() - keyWidth * (blackKeys.length - 1)) / 2 + i * keyWidth;
                addObject(key, x, y);
           }
        }
    }
}